#include "Naming/video/VideoCompany.h"
#include "Naming/video/VideoServiceProvider.h"
#include "Naming/video/VideoSupplier.h"
#include "Naming/video/NamingVideoSupplier.h"

#include <tao/corba.h>
#include <ace/OS.h>

#include "Common/ObjectAdapter.h"


int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  CORBA::Object_var objVar;
  CosNaming::NamingContext_var rootContextVar;
  CosNaming::Name rootName;

  // Initialise the ORB.
  ACE_TRY_NEW_ENV
    {
      orbVar = CORBA::ORB_init (argc, argv ACE_ENV_ARG_PARAMETER);
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: Initialised ORB\n")));
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Cannot initialise ORB\n"));
      exit (-1);
    }
  ACE_ENDTRY;
    
  // Get a reference to the naming service.
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar->resolve_initial_references (ACE_TEXT ("NameService") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      rootContextVar = CosNaming::NamingContext::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to resolve/narrow to the NameService\n"));
      exit (-1);
    }
  ACE_ENDTRY;

  // Create the service provider singleton.
  VideoSupplier::pServiceProvider_ = new VideoServiceProvider (orbVar);

  // Assign a type and id to the service provider object.
  rootName.length (1);
  rootName[0].id = ACE_OS::strdup (ACE_TEXT ("ServiceProvider"));
  rootName[0].kind = ACE_OS::strdup (ACE_TEXT ("server"));

  // Bind the service provider into the root context.
  CORBA::Boolean doRebind = 0;
  ACE_TRY_NEW_ENV
    {
      rootContextVar->bind (rootName, VideoSupplier::pServiceProvider_->getServant ().in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosNaming::NamingContext::AlreadyBound, ab)
    {
      doRebind = 1;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Could not bind\n"));
      exit (-1);
    }
  ACE_ENDTRY;
  
  if (doRebind)
    {
      ACE_TRY_NEW_ENV
        {
          // Catch already bound and rebind on server re-start.
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("WARNING: Rebinding ServiceProvider\n")));
          rootContextVar->rebind (rootName, VideoSupplier::pServiceProvider_->getServant ().in () ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Cound not rebind\n"));
          exit (-1);
        }
      ACE_ENDTRY;
    }

  // Create a few arbitrary categories.
  const char *cats[] = {ACE_TEXT ("Films"),           //0
                        ACE_TEXT ("Sporting Events"), //1
                        ACE_TEXT ("News Broadcasts"), //2
                        ACE_TEXT ("Soap Operas")      //3
                        };

  // Create a few arbitrary videos.
  const char *vids[]= {ACE_TEXT ("Bridge On The River Kwai"), //0
                       ACE_TEXT ("Mutiny On The Bounty"),     //1
                       ACE_TEXT ("Lawrence Of Arabia"),       //2
                       ACE_TEXT ("Star Wars"),                //3
                       ACE_TEXT ("Great Expectations"),       //4
                       ACE_TEXT ("FA Cup Final"),             //5
                       ACE_TEXT ("BBC"),                      //6
                       ACE_TEXT ("CNN"),                      //7
                       ACE_TEXT ("Coronation Street"),        //8
                       ACE_TEXT ("Neighbours"),               //9
                       ACE_TEXT ("Wimbledon")                 //10
                       };

  // Create a few arbitary video suppliers and bind them into the naming service.

  NamingVideoSupplier videosRUs (ACE_TEXT ("Videos R us"));
  videosRUs.addCategory (cats[0]);
  videosRUs.addCategory (cats[1]);
  videosRUs.addVideo (cats[0], vids[0]);
  videosRUs.addVideo (cats[0], vids[1]);
  videosRUs.addVideo (cats[1], vids[5]);
  videosRUs.addVideo (cats[1], vids[10]);
  videosRUs.bind (rootContextVar.in ());

  NamingVideoSupplier videos4Sale (ACE_TEXT ("Videos for Sale"));
  videos4Sale.addCategory (cats[0]);
  videos4Sale.addCategory (cats[1]);
  videos4Sale.addCategory (cats[2]);
  videos4Sale.addVideo (cats[0], vids[2]);
  videos4Sale.addVideo (cats[0], vids[3]);
  videos4Sale.addVideo (cats[0], vids[4]);
  videos4Sale.addVideo (cats[1], vids[10]);
  videos4Sale.addVideo (cats[2], vids[6]);
  videos4Sale.addVideo (cats[2], vids[7]);
  videos4Sale.bind (rootContextVar.in ());

  NamingVideoSupplier videosOnDemand (ACE_TEXT ("Videos on Demand"));
  videosOnDemand.addCategory (cats[2]);
  videosOnDemand.addCategory (cats[3]);
  videosOnDemand.addVideo (cats[2], vids[6]);
  videosOnDemand.addVideo (cats[3], vids[8]);
  videosOnDemand.addVideo (cats[3], vids[9]);
  videosOnDemand.bind (rootContextVar.in ());

  // Accept incoming requests on this server.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Ready...\n")));
  orbVar->run ();

  // Clean up.
  delete VideoSupplier::pServiceProvider_;
  return 0;
}
