#include <CosTradingC.h>

#include "Trading/Mortgage/Mortgage.h"

#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <ace/OS.h>


int
main (int argc, char** argv)
{
  CORBA::ORB_var orbVar;
  CORBA::Object_var objVar;
  CosTrading::Lookup_var lookupVar;
  
  // Initialise the ORB.
  orbVar = CORBA::ORB_init (argc, argv);
  
  // Resolve to the trading service.
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar->resolve_initial_references (ACE_TEXT ("TradingService") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      lookupVar = CosTrading::Lookup::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to resolve/narrow to the trading service\n"));
      exit (-1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to resolve/narrow to the trading service\n")));
      exit (-1);
    }
  ACE_ENDTRY;

  // Set up parameters for query.
  CosTrading::PolicySeq policies;
  CosTrading::Lookup::SpecifiedProps desiredProps;
  CosTrading::OfferSeq_var offersVar;
  CosTrading::PolicyNameSeq_var policiesAppliedVar;
  CosTrading::OfferIterator_var iteratorVar;
  CosTrading::PropertyNameSeq_var propertySeqVar = new CosTrading::PropertyNameSeq ();
  CORBA::String_var strVar = ACE_OS::strdup (ACE_TEXT ("rate > 0.039"));
  
  propertySeqVar->length (1);
  propertySeqVar[(CORBA::ULong)0] = ACE_OS::strdup (ACE_TEXT ("financial_organisation"));
  desiredProps.prop_names (propertySeqVar.in ());

  // Execute the query.
  ACE_TRY_NEW_ENV
    {
      lookupVar->query (ACE_TEXT ("IDL:Mortgage:1.0"),
                        strVar,
                        ACE_TEXT (""),
                        policies,
                        desiredProps,
                        4,
                        offersVar,
                        iteratorVar,
                        policiesAppliedVar
                        ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to execute query\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to execute query\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Display the offers.

  Mortgage_var mortgageVar;
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("%i available banks with %s\n"), offersVar->length (), strVar.in ()));

  for (CORBA::ULong count = 0; count< offersVar->length (); count++)
    {
      const char *value = 0;

      ACE_TRY_NEW_ENV
        {
            if ((*offersVar)[count].properties[0].value >>= value)
              {
                ACE_DEBUG ((LM_INFO, ACE_TEXT ("Financial organisation name: %s\n"), value));
              }
            else
              {
                ACE_DEBUG ((LM_INFO, ACE_TEXT ("WARNING: Could not determine financial organisation name\n")));
              }

            //Get object reference, narrow to the correct type and invoke the operation.
            mortgageVar = Mortgage::_narrow ((*offersVar)[count].reference.in () ACE_ENV_ARG_PARAMETER);
            ACE_TRY_CHECK;
            ACE_DEBUG ((LM_INFO, ACE_TEXT ("Calculating Mortgage payments\n")));
            mortgageVar->calculateMortgage ((CORBA::Long)100000, (CORBA::Short)25 ACE_ENV_ARG_PARAMETER);
            ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to extract offer information\n"));
          exit (1);
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("ERROR: Failed to extract offer information\n")));
          exit (1);
        }
      ACE_ENDTRY;
    }

  // Complete.
  return 0;
}
