#include <CosTradingC.h>
#include <CosTradingS.h>
#include <CosTradingReposC.h>
#include <CosTradingReposS.h>

#include "Trading/Mortgage/Mortgage.h"
#include "Trading/Mortgage/Util.h"

#include <tao/corba.h>
#include <tao/PortableServer/PortableServer.h>
#include <ace/OS.h>

void
exportMortgageOffer (CosTrading::Register_ptr reg,
                     Mortgage_ptr mortgage,
                     const char *name,
                     CORBA::Float rate)
{
  CosTrading::PropertySeq_var propertiesVar = new CosTrading::PropertySeq ();
  propertiesVar->length (2);

  propertiesVar[(CORBA::ULong)0].name = ACE_OS::strdup (ACE_TEXT ("financial_organisation"));
  propertiesVar[(CORBA::ULong)0].value <<= name;
  propertiesVar[(CORBA::ULong)1].name = ACE_OS::strdup (ACE_TEXT ("rate"));
  propertiesVar[(CORBA::ULong)1].value <<= rate;

  reg->_cxx_export (mortgage, ACE_TEXT ("IDL:Mortgage:1.0"), propertiesVar.in ());
}

void
createMortgage (CosTrading::Register_ptr reg,
                PortableServer::POA_ptr poa,
                const char *name,
                CORBA::Float rate)
{
  MortgageImpl *pMortgageImpl;
  CORBA::Object_var mortgageObjVar;
  Mortgage_var mortgageVar;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Exporting offer for %s with APR %.2f\n"), name, rate));
  
  pMortgageImpl = new MortgageImpl (name, rate);
  poa->activate_object (pMortgageImpl);
  mortgageObjVar = poa->servant_to_reference (pMortgageImpl);
  mortgageVar = Mortgage::_narrow (mortgageObjVar.in ());

  exportMortgageOffer (reg, mortgageVar.in (), mortgageVar->name (), mortgageVar->rate ());
}

int
main (int argc, char** argv)
{
  CORBA::ORB_var orbVar;
  Util *pUtil;
  CORBA::Object_var objVar;

  CosTrading::Lookup_var lookupVar;
  CosTrading::TypeRepository_var typeReposVar;
  CosTradingRepos::ServiceTypeRepository_var serviceTypeReposVar;
  CosTrading::Register_var registerVar;

  // Get a reference to the ORB.
  orbVar = CORBA::ORB_init (argc, argv);

  // Create a new utilities instance.
  pUtil = new Util (orbVar);

  // Resolve to the trading service.  
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar->resolve_initial_references (ACE_TEXT ("TradingService") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      lookupVar = CosTrading::Lookup::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      typeReposVar = lookupVar->type_repos (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      serviceTypeReposVar = CosTradingRepos::ServiceTypeRepository::_narrow (typeReposVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      registerVar = lookupVar->register_if (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to resolve/narrow to the trading service\n"));
      exit (-1);
    }
  ACE_ENDTRY;

  // Add the service type to the trading service.

  CosTradingRepos::ServiceTypeRepository::PropStructSeq_var propertiesVar = new CosTradingRepos::ServiceTypeRepository::PropStructSeq ();
  propertiesVar->length (2);

  propertiesVar[(CORBA::ULong)0].name = ACE_OS::strdup (ACE_TEXT ("financial_organisation"));
  propertiesVar[(CORBA::ULong)0].value_type = CORBA::_tc_string;
  propertiesVar[(CORBA::ULong)0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY_READONLY;
  propertiesVar[(CORBA::ULong)1].name = ACE_OS::strdup (ACE_TEXT ("rate"));
  propertiesVar[(CORBA::ULong)1].value_type = CORBA::_tc_float;
  propertiesVar[(CORBA::ULong)1].mode = CosTradingRepos::ServiceTypeRepository::PROP_NORMAL;

  CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq_var superTypesVar = new CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq ();

  ACE_TRY_NEW_ENV
    {
      serviceTypeReposVar->add_type (ACE_TEXT ("IDL:Mortgage:1.0"),
                                     ACE_TEXT ("IDL:Mortgage:1.0"),
                                     propertiesVar.in (),
                                     superTypesVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::ServiceTypeExists, ste)
    {
      // The service type already exists. Assume it has been added from a
      // previous run of this example. Remove the dead offers from it
      // because they are transient and inactive, and export the offers again.
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Service type Mortgage already exists.\n")));
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Purging dead offers.\n")));

      CosTrading::Admin_var adminVar = lookupVar->admin_if ();
      CosTrading::OfferIdSeq_var idsSeqVar;
      CosTrading::OfferIdIterator_var idsIterVar;
      adminVar->list_offers (100, idsSeqVar, idsIterVar);
      for (CORBA::ULong count = 0; count < idsSeqVar->length (); count++)
        {
          registerVar->withdraw (idsSeqVar [count]);
        }
    }
  ACE_ENDTRY;

  // Get a reference to the root POA, and activate the manager.
  PortableServer::POA_var rootPoaVar = pUtil->getRootPOA ();
  PortableServer::POAManager_var rootManagerVar = rootPoaVar->the_POAManager ();
  rootManagerVar->activate ();

  // Create/activate the mortgage CORBA objects.
  createMortgage (registerVar.in (), rootPoaVar.in (), ACE_TEXT ("LloydsTSB"),  (CORBA::Float) 5.9);
  createMortgage (registerVar.in (), rootPoaVar.in (), ACE_TEXT ("Halifax"),    (CORBA::Float) 6.0);
  createMortgage (registerVar.in (), rootPoaVar.in (), ACE_TEXT ("ABBEY"),      (CORBA::Float) 4.5);
  createMortgage (registerVar.in (), rootPoaVar.in (), ACE_TEXT ("NationWide"), (CORBA::Float) 4.0);
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Successfully exported mortgage offers\n")));

  // Wait for requests.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Waiting for incoming requests\n")));
  orbVar->run ();

  // Complete.
  orbVar->destroy ();
  return 0;
}
