#include "VideoSupplier.h"

VideoServiceProvider *VideoSupplier::pServiceProvider_ = 0;

VideoSupplier::VideoSupplier (const char *name, CORBA::Boolean store)
  : VideoCompany (name, ACE_TEXT ("Video Stream Supplier"))
{
  if (store)
    {
      pServiceProvider_->store (this, name);
    }
}

VideoSupplier::~VideoSupplier ()
{
}

/*
 * @func returns the CORBA reference of this object
 * @rdesc the servant object reference
 */
CORBA::Object_var
VideoSupplier::getServant ()
{
  return VideoSupplier::pServiceProvider_->getObjectAdapter ()->getPOA ()->servant_to_reference (this);
}

/*
 * @func Adds a video category
 * @parm | category | the given category
 */
void
VideoSupplier::addCategory (const char *category)
{
  StringArray *array;
  CORBA::Boolean found = 0;

  for (ASA_Iter itb = ASA_Iter (videos_); itb.next (array) != 0; itb.advance ())
    {
      if (!ACE_OS::strcmp (category, (*array)[0]))
        {
          found = 1;
          break;
        }
    }

  if (!found)
    {
      videos_.size (videos_.size () + 1);
      videos_[videos_.size () - 1].size (1);
      videos_[videos_.size () - 1][0] = (char*) category;
   }
}

/*
 * @func Adds a video title
 * @parm | category | the given video category
 * @parm | name | the given name of the video
 */
void
VideoSupplier::addVideo (const char *category, const char *name)
{
  CORBA::Boolean found = 0;
  int count= 0;
  StringArray *array;

  for (ASA_Iter itb = ASA_Iter (videos_); itb.next (array) != 0; itb.advance ())
    {
      if (!ACE_OS::strcmp (category, (*array)[0]))
        {
          found = 1;
           break;
        }
      count++;
    }

  if (found)
    {
      videos_[count].size (videos_[count].size () + 1);
      videos_[count][videos_[count].size () - 1] = (char*) name;
   }
}


/*
 * @func IDL defined funtion that returns a list of video titles for the given
 * category
 * @parm | category | the given category
 * @rdesc unbounded sequence of strings
 */
Server::OfferList *
VideoSupplier::getOffers (const char *category)
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  Server::OfferList *offers = new Server::OfferList ();
  StringArray *array;
  char **offer;

  for (ASA_Iter itb = ASA_Iter (videos_); itb.next (array) != 0; itb.advance ())
    {
      if (!ACE_OS::strcmp (category, (*array)[0]))
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Found offers for %s\n"), (*array)[0]));
          int videos = 0;
          offers->length (array->size () - 1);

          for (StringArray::ITERATOR itb2 = StringArray::ITERATOR (*array); itb2.next (offer) != 0; itb2.advance ())
            {
              if (videos > 0)
                {
                  (*offers)[videos-1] = ACE_OS::strdup (*offer);
                }
              videos++;
            }
          break;
        }
    }

  return offers;
}

#if defined (ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION)
template class ACE_Array<char*>;
template class ACE_Array_Base<char*>;
template class ACE_Array_Iterator<char*>;
template class ACE_Array<StringArray>;
template class ACE_Array_Base<StringArray>;
template class ACE_Array_Iterator<StringArray>;
#endif // ACE_HAS_EXPLICIT_TEMPLATE_INSTANTIATION
